@@ -1,6 +1,7 @@
 import abc
 from typing import ClassVar, Generic, Type, TypeVar, overload
 
+import jax
 import numpy as onp
 from jax import numpy as jnp
 from overrides import EnforceOverrides, final, overrides
@@ -14,8 +15,8 @@ SEGroupType = TypeVar("SEGroupType", bound="SEBase")
 class MatrixLieGroup(abc.ABC, EnforceOverrides):
     """Interface definition for matrix Lie groups."""
 
-    # Class properties
-    # > These will be set in `_utils.register_lie_group()`
+    # Class properties.
+    # > These will be set in `_utils.register_lie_group()`.
 
     matrix_dim: ClassVar[int]
     """Dimension of square matrix output from `.as_matrix()`."""
@@ -29,14 +30,19 @@ class MatrixLieGroup(abc.ABC, EnforceOverrides):
     space_dim: ClassVar[int]
     """Dimension of coordinates that can be transformed."""
 
-    def __init__(self, parameters: hints.Vector):
+    def __init__(
+        # Notes:
+        # - For the constructor signature to be consistent with subclasses, `parameters`
+        #   should be marked as positional-only. But this isn't possible in Python 3.7.
+        # - This method is implicitly overriden by the dataclass decorator and
+        #   should _not_ be marked abstract.
+        self,
+        parameters: hints.Vector,
+    ):
         """Construct a group object from its underlying parameters."""
-
-        # Note that this method is implicitly overriden by the dataclass decorator and
-        # should _not_ be marked abstract.
         raise NotImplementedError()
 
-    # Shared implementations
+    # Shared implementations.
 
     @overload
     def __matmul__(self: GroupType, other: GroupType) -> GroupType:
@@ -54,12 +60,13 @@ class MatrixLieGroup(abc.ABC, EnforceOverrides):
         """
         if isinstance(other, (onp.ndarray, jnp.ndarray)):
             return self.apply(target=other)
-        if isinstance(other, MatrixLieGroup):
+        elif isinstance(other, MatrixLieGroup):
+            assert self.space_dim == other.space_dim
             return self.multiply(other=other)
         else:
-            assert False, "Invalid argument"
+            assert False, f"Invalid argument type for `@` operator: {type(other)}"
 
-    # Factory
+    # Factory.
 
     @classmethod
     @abc.abstractmethod
@@ -82,7 +89,7 @@ class MatrixLieGroup(abc.ABC, EnforceOverrides):
             Group member.
         """
 
-    # Accessors
+    # Accessors.
 
     @abc.abstractmethod
     def as_matrix(self) -> hints.MatrixJax:
@@ -92,22 +99,22 @@ class MatrixLieGroup(abc.ABC, EnforceOverrides):
     def parameters(self) -> hints.Vector:
         """Get underlying representation."""
 
-    # Operations
+    # Operations.
 
     @abc.abstractmethod
     def apply(self: GroupType, target: hints.Vector) -> hints.VectorJax:
-        """Applies the group action.
+        """Applies group action to a point.
 
         Args:
-            target: Vector to transform.
+            target: Point to transform.
 
         Returns:
-            Transformed vector.
+            Transformed point.
         """
 
     @abc.abstractmethod
     def multiply(self: GroupType, other: GroupType) -> GroupType:
-        """Left-multiplies this transformations with another.
+        """Composes this transformation with another.
 
         Returns:
             self @ other
@@ -135,15 +142,16 @@ class MatrixLieGroup(abc.ABC, EnforceOverrides):
 
     @abc.abstractmethod
     def adjoint(self: GroupType) -> hints.MatrixJax:
-        """Computes the adjoint, which transforms tangent vectors between tangent spaces.
+        """Computes the adjoint, which transforms tangent vectors between tangent
+        spaces.
 
         More precisely, for a transform `GroupType`:
         ```
         GroupType @ exp(omega) = exp(Adj_T @ omega) @ GroupType
         ```
 
-        In robotics, typically used for converting twists, wrenches, and Jacobians
-        between our spatial and body representations.
+        In robotics, typically used for transforming twists, wrenches, and Jacobians
+        across different reference frames.
 
         Returns:
             Output. Shape should be `(tangent_dim, tangent_dim)`.
@@ -167,8 +175,9 @@ class MatrixLieGroup(abc.ABC, EnforceOverrides):
 
     @classmethod
     @abc.abstractmethod
-    def sample_uniform(cls: Type[GroupType], key: jnp.ndarray) -> GroupType:
-        """Draw a uniform sample from the group. Translations are in the range [-1, 1].
+    def sample_uniform(cls: Type[GroupType], key: jax.random.KeyArray) -> GroupType:
+        """Draw a uniform sample from the group. Translations (if applicable) are in the
+        range [-1, 1].
 
         Args:
             key: PRNG key, as returned by `jax.random.PRNGKey()`.
@@ -189,9 +198,10 @@ class SEBase(Generic[ContainedSOType], MatrixLieGroup):
     """Base class for special Euclidean groups.
 
     Each SE(N) group member contains an SO(N) rotation, as well as an N-dimensional
-    translation vector."""
+    translation vector.
+    """
 
-    # SE-specific interface
+    # SE-specific interface.
 
     @classmethod
     @abc.abstractmethod
@@ -210,6 +220,14 @@ class SEBase(Generic[ContainedSOType], MatrixLieGroup):
             Constructed transformation.
         """
 
+    @final
+    @classmethod
+    def from_rotation(cls: Type[SEGroupType], rotation: ContainedSOType) -> SEGroupType:
+        return cls.from_rotation_and_translation(
+            rotation=rotation,
+            translation=onp.zeros(cls.space_dim, dtype=rotation.parameters().dtype),
+        )
+
     @abc.abstractmethod
     def rotation(self) -> ContainedSOType:
         """Returns a transform's rotation term."""
@@ -218,12 +236,12 @@ class SEBase(Generic[ContainedSOType], MatrixLieGroup):
     def translation(self) -> hints.Vector:
         """Returns a transform's translation term."""
 
-    # Overrides
+    # Overrides.
 
     @final
     @overrides
     def apply(self, target: hints.Vector) -> hints.VectorJax:
-        return self.rotation() @ target + self.translation()
+        return self.rotation() @ target + self.translation()  # type: ignore
 
     @final
     @overrides
