@@ -1,6 +1,5 @@
-import dataclasses
-
 import jax
+import jax_dataclasses
 import numpy as onp
 from jax import numpy as jnp
 from overrides import overrides
@@ -29,11 +28,15 @@ def _skew(omega: hints.Vector) -> hints.MatrixJax:
     tangent_dim=6,
     space_dim=3,
 )
-@dataclasses.dataclass(frozen=True)
+@jax_dataclasses.pytree_dataclass
 class SE3(_base.SEBase[SO3]):
-    """Special Euclidean group for proper rigid transforms in 3D."""
+    """Special Euclidean group for proper rigid transforms in 3D.
 
-    # SE3-specific
+    Internal parameterization is `(qw, qx, qy, qz, x, y, z)`. Tangent parameterization
+    is `(vx, vy, vz, omega_x, omega_y, omega_z)`.
+    """
+
+    # SE3-specific.
 
     wxyz_xyz: hints.Vector
     """Internal parameters. wxyz quaternion followed by xyz translation."""
@@ -44,7 +47,7 @@ class SE3(_base.SEBase[SO3]):
         trans = jnp.round(self.wxyz_xyz[..., 4:], 5)
         return f"{self.__class__.__name__}(wxyz={quat}, xyz={trans})"
 
-    # SE-specific
+    # SE-specific.
 
     @staticmethod
     @overrides
@@ -63,7 +66,7 @@ class SE3(_base.SEBase[SO3]):
     def translation(self) -> hints.Vector:
         return self.wxyz_xyz[..., 4:]
 
-    # Factory
+    # Factory.
 
     @staticmethod
     @overrides
@@ -74,13 +77,13 @@ class SE3(_base.SEBase[SO3]):
     @overrides
     def from_matrix(matrix: hints.Matrix) -> "SE3":
         assert matrix.shape == (4, 4)
-        # Currently assumes bottom row is [0, 0, 0, 1]
+        # Currently assumes bottom row is [0, 0, 0, 1].
         return SE3.from_rotation_and_translation(
             rotation=SO3.from_matrix(matrix[:3, :3]),
             translation=matrix[:3, 3],
         )
 
-    # Accessors
+    # Accessors.
 
     @overrides
     def as_matrix(self) -> hints.MatrixJax:
@@ -96,7 +99,7 @@ class SE3(_base.SEBase[SO3]):
     def parameters(self) -> hints.Vector:
         return self.wxyz_xyz
 
-    # Operations
+    # Operations.
 
     @staticmethod
     @overrides
@@ -113,10 +116,10 @@ class SE3(_base.SEBase[SO3]):
         use_taylor = theta_squared < get_epsilon(theta_squared.dtype)
 
         # Shim to avoid NaNs in jnp.where branches, which cause failures for
-        # reverse-mode AD
+        # reverse-mode AD.
         theta_squared_safe = jnp.where(
             use_taylor,
-            1.0,  # Any non-zero value should do here
+            1.0,  # Any non-zero value should do here.
             theta_squared,
         )
         del theta_squared
@@ -151,10 +154,10 @@ class SE3(_base.SEBase[SO3]):
         skew_omega = _skew(omega)
 
         # Shim to avoid NaNs in jnp.where branches, which cause failures for
-        # reverse-mode AD
+        # reverse-mode AD.
         theta_squared_safe = jnp.where(
             use_taylor,
-            1.0,  # Any non-zero value should do here
+            1.0,  # Any non-zero value should do here.
             theta_squared,
         )
         del theta_squared
@@ -191,7 +194,7 @@ class SE3(_base.SEBase[SO3]):
 
     @staticmethod
     @overrides
-    def sample_uniform(key: jnp.ndarray) -> "SE3":
+    def sample_uniform(key: jax.random.KeyArray) -> "SE3":
         key0, key1 = jax.random.split(key)
         return SE3.from_rotation_and_translation(
             rotation=SO3.sample_uniform(key0),
