@@ -1,6 +1,5 @@
-import dataclasses
-
 import jax
+import jax_dataclasses
 import numpy as onp
 from jax import numpy as jnp
 from overrides import overrides
@@ -15,11 +14,15 @@ from .utils import get_epsilon, register_lie_group
     tangent_dim=3,
     space_dim=3,
 )
-@dataclasses.dataclass(frozen=True)
+@jax_dataclasses.pytree_dataclass
 class SO3(_base.SOBase):
-    """Special orthogonal group for 3D rotations."""
+    """Special orthogonal group for 3D rotations.
 
-    # SO3-specific
+    Internal parameterization is `(qw, qx, qy, qz)`. Tangent parameterization is
+    `(omega_x, omega_y, omega_z)`.
+    """
+
+    # SO3-specific.
 
     wxyz: hints.Vector
     """Internal parameters. `(w, x, y, z)` quaternion."""
@@ -71,8 +74,8 @@ class SO3(_base.SOBase):
         pitch: hints.Scalar,
         yaw: hints.Scalar,
     ) -> "SO3":
-        """Generates a transform from a set of Euler angles.
-        Uses the ZYX mobile robot convention.
+        """Generates a transform from a set of Euler angles. Uses the ZYX mobile robot
+        convention.
 
         Args:
             roll: X rotation, in radians. Applied first.
@@ -109,8 +112,7 @@ class SO3(_base.SOBase):
         return jnp.roll(self.wxyz, shift=-1)
 
     def as_rpy_radians(self) -> hints.RollPitchYaw:
-        """Computes roll, pitch, and yaw angles.
-        Uses the ZYX mobile robot convention.
+        """Computes roll, pitch, and yaw angles. Uses the ZYX mobile robot convention.
 
         Returns:
             Named tuple containing Euler angles in radians.
@@ -122,8 +124,7 @@ class SO3(_base.SOBase):
         )
 
     def compute_roll_radians(self) -> hints.ScalarJax:
-        """Compute roll angle.
-        Uses the ZYX mobile robot convention.
+        """Compute roll angle. Uses the ZYX mobile robot convention.
 
         Returns:
             Euler angle in radians.
@@ -133,8 +134,7 @@ class SO3(_base.SOBase):
         return jnp.arctan2(2 * (q0 * q1 + q2 * q3), 1 - 2 * (q1 ** 2 + q2 ** 2))
 
     def compute_pitch_radians(self) -> hints.ScalarJax:
-        """Compute pitch angle.
-        Uses the ZYX mobile robot convention.
+        """Compute pitch angle. Uses the ZYX mobile robot convention.
 
         Returns:
             Euler angle in radians.
@@ -144,8 +144,7 @@ class SO3(_base.SOBase):
         return jnp.arcsin(2 * (q0 * q2 - q3 * q1))
 
     def compute_yaw_radians(self) -> hints.ScalarJax:
-        """Compute yaw angle.
-        Uses the ZYX mobile robot convention.
+        """Compute yaw angle. Uses the ZYX mobile robot convention.
 
         Returns:
             Euler angle in radians.
@@ -154,7 +153,7 @@ class SO3(_base.SOBase):
         q0, q1, q2, q3 = self.wxyz
         return jnp.arctan2(2 * (q0 * q3 + q1 * q2), 1 - 2 * (q2 ** 2 + q3 ** 2))
 
-    # Factory
+    # Factory.
 
     @staticmethod
     @overrides
@@ -190,7 +189,7 @@ class SO3(_base.SOBase):
             q = jnp.array([t, m[2, 1] - m[1, 2], m[0, 2] - m[2, 0], m[1, 0] - m[0, 1]])
             return t, q
 
-        # Compute four cases, then pick the most precise one
+        # Compute four cases, then pick the most precise one.
         # Probably worth revisiting this!
         case0_t, case0_q = case0(matrix)
         case1_t, case1_q = case1(matrix)
@@ -212,7 +211,7 @@ class SO3(_base.SOBase):
             jnp.where(cond2, case2_q, case3_q),
         )
 
-        # # We can also choose to branch, but this is slower
+        # We can also choose to branch, but this is slower.
         # t, q = jax.lax.cond(
         #     matrix[2, 2] < 0,
         #     true_fun=lambda matrix: jax.lax.cond(
@@ -232,7 +231,7 @@ class SO3(_base.SOBase):
 
         return SO3(wxyz=q * 0.5 / jnp.sqrt(t))
 
-    # Accessors
+    # Accessors.
 
     @overrides
     def as_matrix(self) -> hints.MatrixJax:
@@ -251,13 +250,13 @@ class SO3(_base.SOBase):
     def parameters(self) -> hints.Vector:
         return self.wxyz
 
-    # Operations
+    # Operations.
 
     @overrides
     def apply(self: "SO3", target: hints.Vector) -> hints.VectorJax:
         assert target.shape == (3,)
 
-        # Compute using quaternion multiplys
+        # Compute using quaternion multiplys.
         padded_target = jnp.zeros(4).at[1:].set(target)
         return (self @ SO3(wxyz=padded_target) @ self.inverse()).wxyz[1:]
 
@@ -289,11 +288,11 @@ class SO3(_base.SOBase):
         use_taylor = theta_squared < get_epsilon(tangent.dtype)
 
         # Shim to avoid NaNs in jnp.where branches, which cause failures for
-        # reverse-mode AD
+        # reverse-mode AD.
         safe_theta = jnp.sqrt(
             jnp.where(
                 use_taylor,
-                0.0,  # Any constant value should do here
+                0.0,  # Any constant value should do here.
                 theta_squared,
             )
         )
@@ -330,22 +329,26 @@ class SO3(_base.SOBase):
         use_taylor = norm_sq < get_epsilon(norm_sq.dtype)
 
         # Shim to avoid NaNs in jnp.where branches, which cause failures for
-        # reverse-mode AD
+        # reverse-mode AD.
         norm_safe = jnp.sqrt(
             jnp.where(
                 use_taylor,
-                1.0,  # Any non-zero value should do here
+                1.0,  # Any non-zero value should do here.
                 norm_sq,
             )
         )
 
+        atan_n_over_w = jnp.arctan2(
+            jnp.where(w < 0, -norm_safe, norm_safe),
+            jnp.abs(w),
+        )
         atan_factor = jnp.where(
             use_taylor,
             2.0 / w - 2.0 / 3.0 * norm_sq / (w ** 3),
             jnp.where(
                 jnp.abs(w) < get_epsilon(w.dtype),
                 jnp.where(w > 0, 1.0, -1.0) * jnp.pi / norm_safe,
-                2.0 * jnp.arctan(norm_safe / w) / norm_safe,
+                2.0 * atan_n_over_w / norm_safe,
             ),
         )
 
@@ -357,7 +360,7 @@ class SO3(_base.SOBase):
 
     @overrides
     def inverse(self: "SO3") -> "SO3":
-        # Negate complex terms
+        # Negate complex terms.
         return SO3(wxyz=self.wxyz * onp.array([1, -1, -1, -1]))
 
     @overrides
@@ -366,8 +369,8 @@ class SO3(_base.SOBase):
 
     @staticmethod
     @overrides
-    def sample_uniform(key: jnp.ndarray) -> "SO3":
-        # Uniformly sample over S^4
+    def sample_uniform(key: jax.random.KeyArray) -> "SO3":
+        # Uniformly sample over S^4.
         # > Reference: http://planning.cs.uiuc.edu/node198.html
         u1, u2, u3 = jax.random.uniform(
             key=key,
